"use client"

import React from 'react'
import { deleteGallery, galleryVilla, syncGallery, updateGallery } from '@/actions/ttd';
import { Button } from '@/components/ui/button';
import { CheckCheckIcon, PlusIcon, RefreshCwIcon, SaveIcon } from 'lucide-react';
import { toast } from 'sonner';
import { ReactSortable } from "react-sortablejs";
import { cn } from '@/lib/utils';
import Image from 'next/image';
import { Checkbox } from '@/components/ui/checkbox';
import AddGallery from './add-gallery';
import DeleteGallery from './delete-gallery';
import { useLoading } from '@/lib/useLoading';
import { useQuery } from '@tanstack/react-query';
import { toasty } from '@/utils/toasty';

interface Props {
    villa: any;
}
const GallerySection = ({ villa }: Props) => {

    const [gallery, setGallery] = React.useState<any[]>([]);
    const [galleryArray, setGalleryArray] = React.useState<string[]>([]);
    const [selectAll, setSelectAll] = React.useState(false);
    const { setLoading } = useLoading()
    const [open, setOpen] = React.useState(false)

    const toggleOpen = () => setOpen(!open)

    const { data, refetch, isPending , isError, error, isRefetching} = useQuery({
        queryKey: ['ttd-gallery', villa._id],
        queryFn: async () => {
            const response = await galleryVilla(villa._id);
            if (!response.success) {
                throw new Error(response.message);
            }
            return response;
        },
        refetchInterval: 60 * 1000,
        refetchOnWindowFocus: false
    });

    React.useEffect(() => {
        if (isError && error) {
            toast.error(error.message || "Failed to fetch data");
        }
    }, [isError, error]);

    React.useEffect(() => {
        if (data) {
            setGallery(data.data || []);
        }
    }, [data])

    const fetchingGallery = async () => {
        toasty(syncGallery(data._id),{
            loading() {
                setLoading(true)
                return "Getting old gallery..."
            },
            success(data) {
                setGallery(data.data || []);
                return data.message
            },
            error(err) {
                return err.message || "Failed to old gallery"
            },
            finally() {
                setLoading(false)
            },
        })
    }
    
    const handleSelectAll = () => {
        const newSelectAll = !selectAll;
    
        setSelectAll(newSelectAll);
        setGalleryArray(newSelectAll ? gallery.map(({ _id }) => _id) : []);
    };
    
    
    const saveReorder = async () => {
        const newGallery = gallery.map((item: any, index) => ({
            image: item.image,
            order: index + 1
        }))
        const bodyData = {
            gallery: newGallery
        }
        setLoading(false)
                toasty(updateGallery(villa._id, bodyData),{
            loading() {
                setLoading(true)
                return "Saving gallery order..."
            },
            success(data) {
                refetch()
                return data.message
            },
            error(err) {
                return err.message || "Failed to save gallery order"
            },
            finally() {
                setLoading(false)
            },
        })
    }

    const handleCheckbox = (id: string, checked: boolean) => {
        setGalleryArray((prev) => 
            checked ? [...prev, id] : prev.filter((item) => item !== id)
        );
    };


    const handleDelete = async () => {
        const body = {
            id: data._id,
            data: galleryArray
        }
        toasty(deleteGallery(body),{
            loading() {
                setLoading(true)
                return `Deleting ${galleryArray.length} gallery...`
            },
            success(data) {
                setGalleryArray([]);
                refetch()
                return data.message
            },
            error(err) {
                return err.message || "Failed to delete gallery"
            },
            finally() {
                setLoading(false)
            },
        })
    }

    return (
        <div className='md:px-6 space-y-6 border rounded-lg py-6 relative'>
            {isRefetching && (
                <div className='absolute inset-0 bg-white/80 flex items-center justify-center z-50 h-full'>
                    <div className='flex items-center'>
                        <RefreshCwIcon className='animate-spin mr-2' />
                        <p className='text-lg font-bold'>Loading...</p>
                    </div>
                </div>
            )}
            <div className='flex items-center justify-between'>
                <Button 
                    size="sm" 
                    type="button" 
                    variant="outline" 
                    onClick={handleSelectAll}
                >
                    <CheckCheckIcon />
                    <span>Select all</span>
                </Button>
                <div className='flex items-center justify-end gap-6'>
                    {galleryArray.length > 0 && (
                        <DeleteGallery 
                            onDelete={() => handleDelete()}
                            title={`Delete (${galleryArray.length}) Gallery`}
                        />
                    )}
                    <Button type="button" size="sm" onClick={toggleOpen}>
                        <PlusIcon />
                        <span>Add</span>
                    </Button>
                    <Button type="button" size="sm" variant="green" onClick={() => refetch()} disabled={isPending}>
                        <RefreshCwIcon />
                        <span>Refresh</span>
                    </Button>
                    <Button type="button" size="sm" variant="semi" onClick={fetchingGallery}>
                        <RefreshCwIcon />
                        <span>Sync Gallery with old System</span>
                    </Button>
                    <Button type="button" size="sm" variant="blue" onClick={saveReorder} disabled={open}>
                        <SaveIcon />
                        <span>Save</span>
                    </Button>
                </div>
            </div>
            <AddGallery
                open={open}
                setOpen={setOpen}
                id={villa._id} 
                name={villa.name} 
                refetch={refetch}
                setLoading={setLoading}
            />
            <div className='border rounded-lg bg-muted p-6 space-y-3'>
                {isPending ? (
                    <div className='flex items-center justify-center h-full'>
                        <div className='flex items-center'>
                            <RefreshCwIcon className='animate-spin mr-2' />
                            <p className='text-lg font-bold'>Loading...</p>
                        </div>
                    </div>
                ) : (
                    <ReactSortable 
                        list={gallery} 
                        setList={setGallery}
                        animation={150}
                        className="grid lg:grid-cols-7 grid-cols-3 gap-4 select-none " // Grid layout
                        chosenClass="choosen"
                    >   
                        {gallery.length > 0 ? 
                            gallery.map((item, index) => (
                                <div key={item._id || index} className={cn("relative p-1 rounded-lg", galleryArray.includes(item._id) ? "bg-gundam-red" : "")}>
                                    <div className='absolute top-1 left-1 z-30 w-[96%] h-1/2 bg-gradient-to-b from-neutral-900 to-transparent rounded-t-lg' />
                                    <Image
                                        className="pointer-events-none object-cover w-56 h-40 rounded-lg"
                                        alt={`gallery-image-${item._id}`}
                                        src={`https://imagedelivery.net/wO6ihUlg1M7aee1mES2PLg/${item.image}/public`}
                                        width={500}
                                        height={500}
                                    />
                                    <div className='absolute top-4 left-4 z-40 bg-gundam-semi-black h-6 w-6 rounded-sm flex items-center justify-center'>
                                        <legend className='text-center text-sm font-bold text-white'>{item.order}</legend>
                                    </div>
                                    <div className='absolute top-4 right-4 z-40'>
                                        <Checkbox 
                                            className="bg-white border-none h-6 w-6 data-[state=checked]:bg-gundam-red" 
                                            checked={galleryArray.includes(item._id)}
                                            onCheckedChange={(checked) => handleCheckbox(item._id, Boolean(checked))} 
                                        />
                                    </div>
                                </div>
                            )) : (
                                <div className='col-span-full'>
                                    <p className='text-center font-bold'>No Galleries Found</p>
                                </div>
                            )
                        }
                    </ReactSortable>
                )}
            </div>
        </div>
    )
}

export default GallerySection